"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const S3 = require("aws-sdk/clients/s3");
const errors_1 = require("../../errors");
const api_1 = require("../../lib/api");
const aws_1 = require("../../lib/aws");
const cordova_1 = require("../../lib/cordova");
const fs_1 = require("../../lib/fs");
const build_1 = require("../../lib/build");
const requiredImages = [
    {
        name: 'snapshot',
        environmentKey: 'SNAPSHOT_IMAGE'
    },
    {
        name: 'package-android',
        environmentKey: 'PACKAGE_ANDROID_IMAGE'
    }
];
async function run(ctx) {
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    await build_1.prepareDockerImages(ctx, requiredImages);
    await build_1.buildApp(ctx, true, false);
    await buildAndroidApp(ctx);
    await uploadArtifact(ctx, s3);
    await build_1.uploadSourceMaps(ctx, s3);
}
exports.run = run;
async function buildAndroidApp(ctx) {
    const dockerImage = ctx.env.require('PACKAGE_ANDROID_IMAGE');
    const appId = ctx.env.require('APP_ID');
    const profileTag = ctx.env.get('CREDENTIAL_PROFILE_TAG');
    let credentials;
    if (profileTag) {
        credentials = (await api_1.getAndroidCredentials(ctx, appId, profileTag)).data;
    }
    await cordova_1.cordovaPlatformAdd(ctx, dockerImage, 'android');
    await cordova_1.cordovaBuildAndroid(ctx, dockerImage, { credentials });
}
exports.buildAndroidApp = buildAndroidApp;
async function uploadArtifact(ctx, s3) {
    const packageBucket = ctx.env.require('PACKAGE_S3_BUCKET');
    // TODO: Send entire key from API in Environment
    // const packageS3Key = ctx.env.require('PACKAGE_S3_KEY');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    const buildType = ctx.env.get('BUILD_TYPE', 'debug');
    const appName = ctx.env.require('APP_NAME');
    const branch = ctx.env.require('BRANCH');
    const sha = ctx.env.require('COMMIT_SHA');
    const apkNameCordova7 = `android-${buildType}.apk`;
    const apkNameCordova8 = `app-${buildType}.apk`;
    const apkPathCordova7 = 'platforms/android/build/outputs/apk';
    const apkPathCordova8 = `platforms/android/app/build/outputs/apk/${buildType}`;
    let artifactPath;
    try {
        [artifactPath] = await fs_1.findFiles(apkPathCordova7, apkNameCordova7);
    }
    catch (e) {
        [artifactPath] = await fs_1.findFiles(apkPathCordova8, apkNameCordova8);
    }
    if (!artifactPath) {
        const msg = (`${ctx.c.red(`Could not find ${apkNameCordova7} in ${apkPathCordova7} or ${apkNameCordova8} in ${apkPathCordova8}`)}\n` +
            `${ctx.c.yellow(`This may be caused by using Crosswalk, which is not supported by Ionic Pro.`)}`);
        throw new errors_1.JobException(msg, 'user');
    }
    const artifact = fs.createReadStream(artifactPath);
    const key = `builds/${snapshotId}-${buildType}.apk`;
    const contentDisposition = `attachment; filename=${appName}-${branch}-${sha}-${buildType}.apk`;
    ctx.serverlog.info({ bucket: packageBucket, key }, `Uploading ${artifactPath} to s3://${packageBucket}/${key}`);
    ctx.userlog.info(ctx.c.yellow('Uploading APK...'));
    await s3.upload(packageBucket, key, artifact, { ContentDisposition: contentDisposition });
    ctx.userlog.info(ctx.c.green('APK uploaded successfully!'));
}
exports.uploadArtifact = uploadArtifact;
