"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const archiver = require("archiver");
const S3 = require("aws-sdk/clients/s3");
const errors_1 = require("../../errors");
const aws_1 = require("../../lib/aws");
const fs_1 = require("../../lib/fs");
const build_1 = require("../../lib/build");
// https://docs.aws.amazon.com/AmazonCloudFront/latest/DeveloperGuide/ServingCompressedFiles.html
const MEDIA_TYPES = {
    '.css': 'text/css',
    '.csv': 'text/csv',
    '.eot': 'application/vnd.ms-fontobject',
    '.gif': 'image/gif',
    '.html': 'text/html',
    '.jpeg': 'image/jpeg',
    '.jpg': 'image/jpeg',
    '.js': 'application/javascript',
    '.json': 'application/json',
    '.map': 'application/json',
    '.mp3': 'audio/mpeg',
    '.mp4': 'video/mp4',
    '.otf': 'font/otf',
    '.png': 'image/png',
    '.svg': 'image/svg+xml',
    '.ts': 'application/typescript',
    '.tsv': 'text/tab-separated-values',
    '.ttf': 'font/ttf',
    '.woff': 'font/woff',
    '.woff2': 'font/woff2',
    '.xml': 'application/xml',
    '.zip': 'application/zip',
};
const requiredImages = [
    {
        name: 'snapshot',
        environmentKey: 'SNAPSHOT_IMAGE'
    }
];
async function run(ctx) {
    const s3 = new aws_1.S3Client(new S3({
        apiVersion: '2006-03-01',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
    }));
    await build_1.prepareDockerImages(ctx, requiredImages);
    await build_1.buildApp(ctx, false, true);
    await uploadApp(ctx, s3);
    await uploadZipApp(ctx, s3);
    await build_1.uploadSourceMaps(ctx, s3);
}
exports.run = run;
async function uploadZipApp(ctx, s3) {
    const wwwpath = path.resolve(ctx.codedir, 'www');
    try {
        await fs_1.fsAccess(wwwpath);
    }
    catch (e) {
        throw new errors_1.JobException('No www directory found. ' +
            'You must either have a build script which produces ' +
            'your app in the www directory or commit the ' +
            'www directory to version control.', 'user');
    }
    const bucket = ctx.env.require('SNAPSHOT_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    ctx.userlog.info(ctx.c.yellow('Uploading build...'));
    try {
        const archive = archiver('zip');
        archive.directory(wwwpath, '');
        archive.finalize();
        await s3.upload(bucket, `${snapshotId}.zip`, archive);
        ctx.userlog.info(ctx.c.green('Build uploaded successfully!'));
    }
    catch (e) {
        ctx.userlog.error(ctx.c.red('Failed to upload build to storage. Please retry your build.'));
        throw e;
    }
}
exports.uploadZipApp = uploadZipApp;
async function uploadApp(ctx, s3) {
    const bucket = ctx.env.require('PRO_DEPLOY_S3_BUCKET');
    const appId = ctx.env.require('APP_ID');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    const wwwpath = path.resolve(ctx.codedir, 'www');
    const f = await fs_1.fsReadFile(path.resolve(wwwpath, 'pro-manifest.json'), { encoding: 'utf8' });
    ctx.userlog.info(ctx.c.yellow('Uploading app files...'));
    JSON.parse(f).forEach(async (fileObject) => {
        try {
            ctx.serverlog.debug(`Uploading ${fileObject.file}...`);
            let newKey = fs.createReadStream(path.resolve(wwwpath, fileObject.href));
            let ext = path.extname(fileObject.href).toLowerCase();
            let params = {
                ContentType: MEDIA_TYPES[ext] || 'application/octet-stream',
            };
            await s3.upload(bucket, `${appId}/${snapshotId}/${fileObject.href}`, newKey, params);
        }
        catch (e) {
            ctx.userlog.error(ctx.c.red('Failed uploading from manifest, please retry your build.'));
            throw e;
        }
    });
    ctx.userlog.info(ctx.c.yellow(`Uploading manifest...`));
    await s3.upload(bucket, `${appId}/${snapshotId}/pro-manifest.json`, fs.createReadStream(path.resolve(wwwpath, 'pro-manifest.json')), { ContentType: 'application/json' });
    ctx.userlog.info(ctx.c.green('Manifest uploaded successfully!'));
}
exports.uploadApp = uploadApp;
