"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const glob = require("glob");
const archiver = require("archiver");
const npm_1 = require("./npm");
const project_1 = require("./project");
const app_analytics_1 = require("./app-analytics");
const git_1 = require("./git");
const ECR = require("aws-sdk/clients/ecr");
const docker_1 = require("./docker");
const aws_1 = require("./aws");
const shell_1 = require("./shell");
const path = require("path");
const fs_1 = require("../lib/fs");
async function buildApp(ctx, alwaysInstall = false, generateManifest = false) {
    const image = ctx.env.require('SNAPSHOT_IMAGE');
    const packageJson = await npm_1.readPackageJson(ctx.codedir, 'user');
    const dockerEnv = {
        APP_ID: ctx.env.require('APP_ID'),
        CI: ctx.env.get('CI', '1'),
        IONIC_CONFIG_DIRECTORY: `/usr/src/app/.ionic`,
        npm_config_proxy: ctx.env.get('npm_config_proxy', ''),
        npm_config_registry: ctx.env.get('npm_config_registry', '')
    };
    ctx.userlog.info(ctx.c.magenta('Parsing ionic.config.json for project settings'));
    const project = await project_1.readProjectFile(ctx);
    ctx.userlog.info(ctx.c.magenta('Project Settings:'));
    ctx.userlog.info(ctx.c.magenta(`\tName: ${project.name}`));
    ctx.userlog.info(ctx.c.magenta(`\tApp ID: ${project.app_id}`));
    ctx.userlog.info(ctx.c.magenta(`\tType: ${project.type}`));
    try {
        await app_analytics_1.scrapeApp(project, packageJson, ctx.codedir);
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
    try {
        await git_1.writeGitCredentials(ctx);
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
    const shouldAttemptBuild = packageJson.scripts && packageJson.scripts.build;
    if (shouldAttemptBuild || alwaysInstall) {
        // Install Required Deps
        await docker_1.prettyUserCommand(ctx, image, ['npm', 'install', '--quiet', '--no-optional'], dockerEnv);
        ctx.userlog.info(ctx.c.magenta('\n\n****Printing Build Environment Details****\n\n'));
        await docker_1.prettyUserCommand(ctx, image, ['ionic', 'info'], dockerEnv);
        if (shouldAttemptBuild) {
            // Build app
            ctx.userlog.info(ctx.c.yellow('Build script detected in package.json...'));
            await docker_1.prettyUserCommand(ctx, image, ['npm', 'run', 'build'], dockerEnv);
        }
    }
    else {
        ctx.userlog.info(ctx.c.yellow('No build script detected in package.json. Skipping install and build steps.'));
    }
    if (generateManifest) {
        // Generate app manifest
        ctx.userlog.info(ctx.c.yellow('Generating app manifest...'));
        await docker_1.prettyUserCommand(ctx, image, ['ionic', 'deploy', 'manifest'], dockerEnv);
    }
}
exports.buildApp = buildApp;
async function prepareDockerImages(ctx, images) {
    const ecr = new aws_1.ECRClient(new ECR({
        apiVersion: '2015-09-21',
        accessKeyId: ctx.env.require('AWS_ACCESS_KEY_ID'),
        secretAccessKey: ctx.env.require('AWS_SECRET_ACCESS_KEY'),
        sessionToken: ctx.env.require('AWS_SESSION_TOKEN'),
        region: ctx.env.require('AWS_REGION'),
    }));
    const token = await ecr.getLoginCredentials();
    await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['login', '--username', token.username, '--password', token.password, token.endpoint], {});
    images.forEach(async (image) => await fetchOrBuildImage(ctx, await getImageInfo(ctx, image), ecr));
}
exports.prepareDockerImages = prepareDockerImages;
async function fetchOrBuildImage(ctx, image, ecr) {
    try {
        if (image.buildDir) {
            ctx.serverlog.info(`Building ${image.url}...`);
            await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['build', '-t', image.url, image.buildDir], {});
        }
        else {
            ctx.serverlog.info(`Pulling ${image.url}...`);
            await shell_1.runcmd({ outlog: ctx.serverlog }, 'docker', ['pull', image.url], {});
        }
    }
    catch (e) {
        ctx.serverlog.error(e);
    }
}
exports.fetchOrBuildImage = fetchOrBuildImage;
async function getImageInfo(ctx, image) {
    const environment = ctx.env.require('ENVIRONMENT');
    const dir = path.dirname(path.dirname(ctx.bindir));
    const imageInfo = { url: ctx.env.require(image.environmentKey) };
    if (environment === 'DEVELOPMENT') {
        imageInfo.buildDir = path.resolve(dir, 'docker', image.name);
    }
    return imageInfo;
}
exports.getImageInfo = getImageInfo;
async function uploadSourceMaps(ctx, s3) {
    const mapPath = path.resolve(ctx.codedir, '.sourcemaps');
    try {
        await fs_1.fsAccess(mapPath);
    }
    catch (e) {
        ctx.userlog.warn(ctx.c.red('No .sourcemaps directory found. Skipping sourcemaps upload.'));
        return;
    }
    const bucket = ctx.env.require('SNAPSHOT_S3_BUCKET');
    const snapshotId = ctx.env.require('SNAPSHOT_ID');
    ctx.userlog.info(ctx.c.yellow('Uploading sourcemaps...'));
    // upload zipped version of the sourcemaps
    try {
        const archive = archiver('zip');
        archive.glob('*.map', { cwd: mapPath });
        archive.finalize();
        await s3.upload(bucket, `${snapshotId}/sourcemaps.zip`, archive);
        ctx.userlog.info(ctx.c.green('Sourcemaps zip archive uploaded successfully!'));
    }
    catch (e) {
        ctx.userlog.error(ctx.c.red('Failed to upload sourcemaps to storage. Please retry your build.'));
        throw e;
    }
    // upload single sourcemaps files
    const mapFiles = await new Promise((resolve, reject) => {
        const items = [];
        new glob.Glob(path.join(mapPath, '*.map'), { dot: true })
            .on('error', err => {
            ctx.userlog.error(ctx.c.red('Failed to scan sourcemaps directory.'));
        })
            .on('match', item => items.push(item))
            .on('end', () => resolve(items));
    });
    ctx.userlog.info(ctx.c.green('Uploading the following sourcemap files:'));
    for (let mapFilePath of mapFiles) {
        const params = {
            ContentType: 'application/json',
        };
        let fileName = path.basename(mapFilePath);
        ctx.userlog.info(ctx.c.green(`  ${fileName}`));
        try {
            let newKey = fs.createReadStream(mapFilePath);
            await s3.upload(bucket, `${snapshotId}/sourcemaps/${fileName}`, newKey, params);
        }
        catch (e) {
            ctx.userlog.error(ctx.c.red(`  Failed uploading sourcemap file ${fileName}`));
        }
    }
}
exports.uploadSourceMaps = uploadSourceMaps;
